from __future__ import absolute_import
from django.utils.version import get_complete_version
from django.contrib.auth.backends import RemoteUserBackend
from django_webtest.compat import from_wsgi_safe_string

class WebtestUserBackend(RemoteUserBackend):
    """ Auth backend for django-webtest auth system """

    # Although it appears that we're just creating work for ourselves here
    # because we're calling super(WebtestUserBackend, self).authenticate with
    # unchanged arguments, what matters is the name of the arguments.
    # Specifically, django.contrib.auth.authenticate looks at the names of the
    # arguments of a backend's authenticate method to decide whether the
    # backend can be used to authenticate a particular request.
    if get_complete_version() >= (1, 11):
        def authenticate(self, request, django_webtest_user):
            return super(WebtestUserBackend, self).authenticate(
                request, django_webtest_user)
    else:
        def authenticate(self, django_webtest_user):
            return super(WebtestUserBackend, self).authenticate(
                django_webtest_user)

    def clean_username(self, username):
        return from_wsgi_safe_string(username)


class WebtestUserWithoutPermissionsBackend(WebtestUserBackend):
    """
    Auth backend that passes-through any permission check to further backends
    """

    def get_perm(self, user_obj, perm, obj=None):
        # Indicate that this backend does not handle permissions and
        # allow Django's django.contrib.auth.models._user_has_perm
        # utility to move on to other enabled authentication backends.
        return False



# -*- coding: utf-8 -*-
from django_webtest import WebTestMixin
import pytest


class MixinWithInstanceVariables(WebTestMixin):
    """
    Override WebTestMixin to make all of its variables instance variables
    not class variables; otherwise multiple django_app_factory fixtures contend
    for the same class variables
    """
    def __init__(self):
        self.extra_environ = {}
        self.csrf_checks = True
        self.setup_auth = True


@pytest.fixture(scope='session')
def django_app_mixin():
    app_mixin = MixinWithInstanceVariables()
    return app_mixin


@pytest.fixture
def django_app(django_app_mixin):
    django_app_mixin._patch_settings()
    django_app_mixin.renew_app()
    yield django_app_mixin.app
    django_app_mixin._unpatch_settings()


@pytest.fixture
def django_app_factory():
    app_mixin = MixinWithInstanceVariables()

    def factory(csrf_checks=True, extra_environ=None):
        app_mixin.csrf_checks = csrf_checks
        if extra_environ:
            app_mixin.extra_environ = extra_environ
        app_mixin._patch_settings()
        app_mixin.renew_app()
        return app_mixin.app

    yield factory
    
    app_mixin._unpatch_settings()



# -*- coding: utf-8 -*-
import django
from django.contrib.auth.middleware import RemoteUserMiddleware
from django.core.exceptions import ImproperlyConfigured
from django.contrib import auth
if django.VERSION >= (1, 10):
    from django.utils.deprecation import MiddlewareMixin
else:
    MiddlewareMixin = object

from django_webtest.compat import is_authenticated


class WebtestUserMiddleware(RemoteUserMiddleware):
    """
    Middleware for utilizing django-webtest simplified auth
    ('user' arg for self.app.post and self.app.get).

    Mostly copied from RemoteUserMiddleware, but the auth backend is changed
    (by changing ``auth.authenticate`` arguments) in order to keep
    RemoteUser backend untouched during django-webtest auth.
    """

    header = "WEBTEST_USER"

    def __call__(self, request):
        # AuthenticationMiddleware is required so that request.user exists.
        if not hasattr(request, 'user'):
            raise ImproperlyConfigured(
                "The django-webtest auth middleware requires the "
                "'django.contrib.auth.middleware.AuthenticationMiddleware' "
                "to be installed. Add it to your MIDDLEWARE setting "
                "or disable django-webtest auth support by setting "
                "'setup_auth' property of your WebTest subclass to False."
            )
        try:
            username = request.META[self.header]
        except KeyError:
            # If specified header doesn't exist then return (leaving
            # request.user set to AnonymousUser by the
            # AuthenticationMiddleware).
            return self.get_response(request)
        # If the user is already authenticated and that user is the user we are
        # getting passed in the headers, then the correct user is already
        # persisted in the session and we don't need to continue.
        if is_authenticated(request.user):
            if hasattr(request.user, "get_username"):
                authenticated_username = request.user.get_username()
            else:
                authenticated_username = request.user.username
            clean_username = self.clean_username(username, request)
            if authenticated_username == clean_username:
                return self.get_response(request)
        # We are seeing this user for the first time in this session, attempt
        # to authenticate the user.
        user = auth.authenticate(request=request, django_webtest_user=username)
        if user:
            # User is valid.  Set request.user and persist user in the session
            # by logging the user in.
            request.user = user
            auth.login(request, user)
        return self.get_response(request)


class DisableCSRFCheckMiddleware(MiddlewareMixin):
    def process_request(self, request):
        request._dont_enforce_csrf_checks = True



import os
import sys
import django

PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))
join = lambda p: os.path.abspath(os.path.join(PROJECT_ROOT, p))

# TODO configure pytest testpaths instead of doing this
sys.path.insert(0, join('..'))

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': join('db.sqlite'),
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

if os.environ.get('USE_POSTGRES'):
    DATABASES['default']['ENGINE'] = 'django.db.backends.postgresql_psycopg2'
    DATABASES['default']['HOST'] = os.environ.get('POSTGRES_HOST', '')
    DATABASES['default']['PORT'] = int(os.environ.get('POSTGRES_PORT', '5432'))
    DATABASES['default']['USER'] = os.environ.get('POSTGRES_USER', '')
    DATABASES['default']['PASSWORD'] = os.environ.get('POSTGRES_PASSWORD', '')
    DATABASES['default']['NAME'] = os.environ.get(
            'POSTGRES_DB', 'django_webtest_tests'
    )

SITE_ID = 1
ROOT_URLCONF = 'urls'
SECRET_KEY = '5mcs97ar-(nnxhfkx0%^+0^sr!e(ax=x$2-!8dqy25ff-l1*a='
DEBUG = False

USE_I18N = True
USE_L10N = True
TIME_ZONE = 'America/Chicago'
LANGUAGE_CODE = 'en-us'

MEDIA_ROOT = join('media')
MEDIA_URL = '/media/'
STATIC_URL = '/static/'
LOGIN_REDIRECT_URL = '/template/index.html'


TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'debug': DEBUG,
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]


if django.VERSION < (1, 10):
    TEMPLATE_DEBUG = DEBUG

    TEMPLATE_LOADERS = (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
    )

    TEMPLATE_DIRS = (
        join('templates'),
    )


MIDDLEWARE = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'testapp_tests.middleware.UserMiddleware',
)

if django.VERSION < (1, 10):
    MIDDLEWARE_CLASSES = MIDDLEWARE
    del MIDDLEWARE


INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django_webtest',
    'django_webtest_tests',
    'django_webtest_tests.testapp_tests',
)

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'



import django

if django.VERSION >= (1, 5):
    from django.contrib.auth.models import AbstractBaseUser
    from django.db import models


    class MyCustomUser(AbstractBaseUser):
        email = models.EmailField(
            max_length=255,
            unique=True,
            db_index=True,
        )
        USERNAME_FIELD = 'email'

        class Meta:
            app_label = "testapp_tests"



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import django
if django.VERSION >= (1, 10):
    from django.utils.deprecation import MiddlewareMixin
else:
    MiddlewareMixin = object

class UserMiddleware(MiddlewareMixin):
    def process_request(self, request):
        request.user.processed = True



# -*- coding: utf-8 -*-
from django.test import Client
from django.http import SimpleCookie
from webtest import TestResponse
from django_webtest.compat import urlparse


class DjangoWebtestResponse(TestResponse):
    """
    WebOb's Response quacking more like django's HttpResponse.

    This is here to make more django's TestCase asserts work,
    not to provide a generally useful proxy.
    """
    streaming = False

    @property
    def status_code(self):
        return self.status_int

    @property
    def _charset(self):
        return self.charset

    @property
    def content(self):
        return self.body

    @property
    def url(self):
        return self['location']

    @property
    def client(self):
        client = Client()
        client.cookies = SimpleCookie()
        for k, v in self.test_app.cookies.items():
            client.cookies[k] = v
        return client

    def __getitem__(self, item):
        item = item.lower()
        if item == 'location':
            # django's test response returns location as http://testserver/,
            # WebTest returns it as http://localhost:80/
            scheme, netloc, path, query, fragment = urlparse.urlsplit(
                self.location)
            if netloc == 'localhost:80':
                netloc = 'testserver'
            return urlparse.urlunsplit(
                [scheme, netloc, path, query, fragment])
        for header, value in self.headerlist:
            if header.lower() == item:
                return value
        raise KeyError(item)



# -*- coding: utf-8 -*-

from django.conf import settings

initial_settings = {
    "DEBUG_PROPAGATE_EXCEPTIONS": settings.DEBUG_PROPAGATE_EXCEPTIONS,
    "AUTHENTICATION_BACKENDS": settings.AUTHENTICATION_BACKENDS,
}

def test_django_app(django_app):
    resp = django_app.get('/')
    assert resp.status_int == 200


def test_django_app_post(django_app_factory):
    app = django_app_factory(csrf_checks=False)
    resp = app.post('/')
    assert resp.status_int == 200

def test_app_factory():
    """Ensure django_app_factory properly resets settings."""
    assert settings.DEBUG_PROPAGATE_EXCEPTIONS is initial_settings["DEBUG_PROPAGATE_EXCEPTIONS"]
    assert settings.AUTHENTICATION_BACKENDS == initial_settings["AUTHENTICATION_BACKENDS"]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import django
from django.contrib.auth.models import User

try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse

from unittest import skipIf
from unittest.mock import patch

from django.test.testcases import override_settings
from webtest import AppError, TestApp

from django_webtest import WebTest
from django_webtest.compat import is_anonymous, is_authenticated


class MethodsTest(WebTest):

    csrf_checks = False

    def assertMethodWorks(self, meth, name):
        response = meth('/')
        self.assertEqual(response.status_int, 200)
        response.mustcontain(name)
        #self.assertTrue(name in response)

    def assertMethodWorksXHR(self, meth, name):
        try:
            response = meth('/', xhr=True)
        except TypeError as e:
            # for webtest < 2
            self.assertIn('xhr', e.message)
        else:
            # for webtest == 2
            self.assertEqual(response.status_int, 200)
            response.mustcontain(name)

    def test_get(self):
        self.assertMethodWorks(self.app.get, 'GET')

    def test_post(self):
        self.assertMethodWorks(self.app.post, 'POST')

    def test_put(self):
        self.assertMethodWorks(self.app.put, 'PUT')

    def test_delete(self):
        self.assertMethodWorks(self.app.delete, 'DELETE')

    def test_get_json(self):
        self.assertMethodWorks(self.app.get, 'GET')

    def test_post_json(self):
        self.assertMethodWorks(self.app.post_json, 'POST')

    def test_put_json(self):
        self.assertMethodWorks(self.app.put_json, 'PUT')

    def test_delete_json(self):
        self.assertMethodWorks(self.app.delete_json, 'DELETE')

    def test_get_xhr(self):
        self.assertMethodWorksXHR(self.app.get, 'GET')

    def test_post_xhr(self):
        self.assertMethodWorksXHR(self.app.post, 'POST')

    def test_put_xhr(self):
        self.assertMethodWorksXHR(self.app.put, 'PUT')

    def test_delete_xhr(self):
        self.assertMethodWorksXHR(self.app.delete, 'DELETE')

    if hasattr(TestApp, 'patch'):  # old WebTest versions don't have 'patch' method
        def test_patch(self):
            self.assertMethodWorks(self.app.patch, 'PATCH')

    def test_head(self):
        response = self.app.head('/')
        self.assertEqual(response.status_int, 200)
        expected = b'' if django.VERSION < (1, 10) else b'HEAD'
        self.assertEqual(response.body, expected)

    def test_options(self):
        self.assertMethodWorks(self.app.options, 'OPTIONS')

    def test_get_auto_follow_default(self):
        response = self.app.get(reverse('remove-prefix-redirect',
                                        args=('template/index.html',)))
        self.assertEqual(response.status_code, 302)

    def test_get_auto_follow_true(self):
        response = self.app.get(reverse('remove-prefix-redirect',
                                        args=('template/index.html',)),
                                auto_follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Hello')

    def test_get_auto_follow_and_expect_errors(self):
        response = self.app.get(reverse('remove-prefix-redirect',
                                        args=('some-404/',)),
                                auto_follow=True, expect_errors=True)
        self.assertEqual(response.status_code, 404)

    def test_get_with_positional_args(self):
        response = self.app.get('/', {})
        self.assertEqual(response.status_code, 200)


class PostRequestTest(WebTest):
    csrf_checks = False

    def test_post_request(self):
        response = self.app.post('/')
        self.assertEqual(response.status_int, 200)
        self.assertTrue('POST' in response)

    def test_404_response(self):
        self.assertRaises(AppError, self.app.get, '/404/')


class CsrfProtectionTest(WebTest):
    def test_csrf_failed(self):
        response = self.app.post('/', expect_errors=True)
        self.assertEqual(response.status_int, 403)


class FormSubmitTest(WebTest):

    def test_form_submit(self):
        page = self.app.get(reverse('check_password'))
        page.form['password'] = 'bar'
        page_with_errors = page.form.submit()

        assert 'Incorrect password' in page_with_errors

        page_with_errors.form['password'] = 'foo'
        page_with_errors.form.submit().follow() # check for 302 response


class GetFormSubmitTest(WebTest):

    def test_form_submit(self):
        page = self.app.get(reverse('search'))
        page.form['q'] = 'bar'
        response = page.form.submit()
        self.assertEqual(response.context['q'], 'bar')


class TemplateContextTest(WebTest):
    def test_rendered_templates(self):
        response = self.app.get('/template/index.html')
        self.assertTrue(hasattr(response, 'context'))
        self.assertTrue(hasattr(response, 'template'))

        self.assertEqual(response.template.name, 'index.html')
        self.assertEqual(response.context['bar'], True)
        self.assertEqual(response.context['spam'], None)
        self.assertRaises(KeyError, response.context.__getitem__, 'invalid')

    def test_multiple_templates(self):
        response = self.app.get('/template/complex.html')
        self.assertEqual(len(response.template), 4)
        self.assertEqual(response.template[0].name, 'complex.html')
        self.assertEqual(response.template[1].name, 'include.html')
        self.assertEqual(response.template[2].name, 'include.html')
        self.assertEqual(response.template[3].name, 'include.html')

        self.assertEqual(response.context['foo'], ('a', 'b', 'c'))
        self.assertEqual(response.context['bar'], True)
        self.assertEqual(response.context['spam'], None)


class BaseAuthTest(WebTest):

    def setUp(self):
        self.user = User.objects.create_user('foo', 'example@example.com', '123')

    def _login(self, username, password):
        form = self.app.get(reverse('auth_login')).form
        form['username'] = username
        form['password'] = password
        return form.submit()

    def assertCanLogin(self, user):
        response = self.app.get('/template/index.html', user=user)
        res_user = response.context['user']
        assert is_authenticated(res_user)

        if isinstance(user, User):
            self.assertEqual(res_user, user)
        else:
            self.assertEqual(res_user.username, user)


class AuthTest(BaseAuthTest):

    def test_not_logged_in(self):
        response = self.app.get('/template/index.html')
        user = response.context['user']
        assert not is_authenticated(user)

    def test_logged_using_username(self):
        self.assertCanLogin('foo')

    def test_logged_using_native_username(self):
        self.assertCanLogin(str('foo'))

    def test_logged_using_unicode_username(self):
        self.assertCanLogin('ƒøø')

    def test_logged_using_instance(self):
        self.assertCanLogin(self.user)

    def test_logged_using_unicode_instance(self):
        user = User.objects.create_user('ƒøø', 'example@example.com', '123')
        self.assertCanLogin(user)

    def test_auth_is_enabled(self):
        from django.conf import settings

        auth_middleware = 'django_webtest.middleware.WebtestUserMiddleware'
        assert auth_middleware in self.settings_middleware
        assert 'django_webtest.backends.WebtestUserBackend' in settings.AUTHENTICATION_BACKENDS

        dependency_index = self.settings_middleware.index(
            'django.contrib.auth.middleware.AuthenticationMiddleware')

        self.assertEqual(
            self.settings_middleware.index(auth_middleware),
            dependency_index + 1,
        )

    def test_custom_middleware(self):
        response = self.app.get('/template/index.html', user=self.user)
        user = response.context['user']
        self.assertTrue(user.processed)

    def test_standard_auth(self):
        resp = self._login(self.user.username, '123').follow()
        user = resp.context['user']
        self.assertEqual(user, self.user)

        self.assertIn('sessionid', self.app.cookies)

        resp = self.app.get(reverse('protected'))
        self.assertEqual(resp.status_int, 200)
        resp.mustcontain('ok: {0}'.format(self.user.username))

    def test_reusing_custom_user(self):
        if django.VERSION >= (1, 5):
            from django_webtest_tests.testapp_tests.models import MyCustomUser
            with self.settings(AUTH_USER_MODEL = 'testapp_tests.MyCustomUser'):
                custom_user = MyCustomUser.objects.create(
                        email="custom@example.com")
                custom_user.set_password("123")
                custom_user.save()

                # Let the middleware logs the user in
                self.app.get('/template/index.html', user=custom_user)

                # Middleware authentication check shouldn't crash
                response = self.app.get('/template/index.html',
                                        user=custom_user)
                user = response.context['user']
                assert is_authenticated(user)
                self.assertEqual(user, custom_user)

    def test_normal_user(self):
        """Make sure the fix for custom users in django 1.5 doesn't break
        normal django users"""
        self.app.get('/template/index.html', user=self.user)
        self.app.get('/template/index.html', user=self.user)


@override_settings(
    WEBTEST_AUTHENTICATION_BACKEND=(
        'django_webtest.backends.WebtestUserWithoutPermissionsBackend'),
    AUTHENTICATION_BACKENDS=['testapp_tests.SimpleBackend'],
)
class CustomAuthTest(BaseAuthTest):

    def test_backend_is_enabled(self):
        from django.conf import settings

        assert len(settings.AUTHENTICATION_BACKENDS) == 2
        assert (
            settings.AUTHENTICATION_BACKENDS[0]
            == 'django_webtest.backends.WebtestUserWithoutPermissionsBackend'
        )

    def test_permission_passthrough(self):
        from django.contrib.auth.backends import ModelBackend

        class SimpleBackend(ModelBackend):
            def has_perm(self, user_obj, perm, obj=None):
                return perm == 'tests.allow'

        with patch('testapp_tests.SimpleBackend', SimpleBackend, create=True):
            assert self.user.has_perm('tests.allow') is True
            assert self.user.has_perm('tests.deny') is False


class GlobalAuthTest(BaseAuthTest):

    def test_set_user(self):
        self.app.set_user(self.user.username)
        environ = self.app.extra_environ
        self.assertEqual(environ['WEBTEST_USER'], self.user.username)

        resp = self.app.get('/template/index.html')

        environ = resp.request.environ
        self.assertEqual(environ['WEBTEST_USER'], self.user.username)
        resp.mustcontain('User: {}'.format(self.user.username))

    def test_set_user_reset(self):
        self.app.set_user(self.user.username)
        self.app.set_user(None)
        environ = self.app.extra_environ
        self.assertNotIn('WEBTEST_USER', environ)

        resp = self.app.get('/template/index.html')

        environ = resp.request.environ
        self.assertNotIn('WEBTEST_USER', environ)
        resp.mustcontain('User: AnonymousUser')

    def test_user_param(self):
        resp = self.app.get('/template/index.html', user='bob_morane')

        self.assertEqual(resp.request.environ['WEBTEST_USER'], 'bob_morane')

        resp.mustcontain('User: bob_morane')

    def test_user_param_reset(self):
        self.app.set_user(self.user.username)
        resp = self.app.get('/template/index.html', user=None)

        # this request had no user
        self.assertEqual(resp.request.environ['WEBTEST_USER'], '')
        resp.mustcontain('User: AnonymousUser')
        # app object is unchanged
        self.assertEqual(self.app.extra_environ['WEBTEST_USER'],
                         self.user.username)


class EnvironTest(BaseAuthTest):

    extra_environ = {'REMOTE_ADDR': '127.0.0.2'}

    def test_extra_environ_reset(self):
        resp = self.app.get('/template/index.html', user=self.user)
        environ = resp.request.environ
        self.assertEqual(environ['WEBTEST_USER'], 'foo')
        self.assertEqual(environ['REMOTE_ADDR'], '127.0.0.2')

        resp2 = self.app.get('/template/index.html')
        environ = resp2.request.environ
        self.assertNotIn('WEBTEST_USER', environ)
        self.assertEqual(environ['REMOTE_ADDR'], '127.0.0.2')

        resp3 = self.app.get('/template/index.html',
                             extra_environ={'REMOTE_ADDR': '127.0.0.1'})
        environ = resp3.request.environ
        self.assertEqual(environ['REMOTE_ADDR'], '127.0.0.1')


class RenewAppTest(BaseAuthTest):

    def test_renew_app(self):
        self._login(self.user.username, '123').follow()

        # auth cookie is preserved between self.app.get calls
        page1 = self.app.get('/template/form.html')
        self.assertEqual(page1.context['user'], self.user)

        self.renew_app()

        # cookies were dropped
        page2 = self.app.get('/template/form.html')
        self.assertTrue(is_anonymous(page2.context['user']))

        # but cookies are still there while browsing from stored page
        page1_1 = page1.click('Login')
        self.assertEqual(page1_1.context['user'], self.user)


class DjangoAssertsTest(BaseAuthTest):

    def test_assert_template_used(self):
        response = self.app.get('/template/index.html')
        self.assertTemplateUsed(response, 'index.html')
        self.assertTemplateNotUsed(response, 'complex.html')

        complex_response = self.app.get('/template/complex.html')
        self.assertTemplateUsed(complex_response, 'complex.html')
        self.assertTemplateUsed(complex_response, 'include.html')
        self.assertTemplateNotUsed(complex_response, 'foo.html')

    # def test_assert_form_error(self):
    #     page = self.app.get(reverse('check_password'))
    #     page.form['password'] = 'bar'
    #     page_with_errors = page.form.submit()
    #     self.assertFormError(page_with_errors, 'password', 'Incorrect password.')

    def test_assert_contains(self):
        response = self.app.get('/template/index.html')
        self.assertContains(response, 'Hello', 1)
        self.assertNotContains(response, 'Good bye!')

    def test_assert_contains_unicode(self):
        response = self.app.get('/template/index.html')
        self.assertContains(response, 'привет', 2)

    def test_assert_redirects(self):
        page = self.app.get(reverse('check_password'))
        page.form['password'] = 'foo'
        resp = page.form.submit()
        self.assertRedirects(resp, '/')

    def test_redirects_noauth(self):
        self.app.get(reverse('redirect-to-protected')).follow(status=302)

    def test_redirects(self):
        self.app.get(reverse('redirect-to-protected'), user=self.user).follow()

    def test_assert_redirects_auth(self):
        page = self.app.get(reverse('redirect-to-protected'), user=self.user)
        self.assertRedirects(page, reverse('protected'))


class DisableAuthSetupTest(WebTest):
    setup_auth = False

    def test_no_auth(self):
        from django.conf import settings
        middleware = getattr(settings, 'MIDDLEWARE', None) or settings.MIDDLEWARE_CLASSES
        assert 'django_webtest.middleware.WebtestUserMiddleware' not in middleware
        assert 'django_webtest.backends.WebtestUserBackend' not in settings.AUTHENTICATION_BACKENDS


class TestSession(WebTest):

    def test_session_not_set(self):
        response = self.app.get('/')
        self.assertEqual(response.status_int, 200)
        self.assertEqual({}, self.app.session)

    def test_sessions_disabled(self):
        from django.conf import settings

        apps = list(settings.INSTALLED_APPS)
        apps.remove("django.contrib.sessions")
        settings.INSTALLED_APPS = apps

        response = self.app.get('/')
        self.assertEqual(response.status_int, 200)
        self.assertEqual({}, self.app.session)

    def test_session_not_empty(self):
        self.app.get(reverse('set_session'))
        self.assertEqual('foo', self.app.session['test'])


class TestHeaderAccess(WebTest):
    def test_headers(self):
        response = self.app.get('/')
        self.assertEqual(response['Content-Type'], 'text/html; charset=utf-8')
        self.assertEqual(response['content-type'], 'text/html; charset=utf-8')

    def test_bad_header(self):
        def access_bad_header():
            response = self.app.get('/')
            response['X-Unknown-Header']
        self.assertRaises(KeyError, access_bad_header)


class TestCookies(WebTest):
    def test_cookies(self):
        self.app.set_cookie(str('test_cookie'), str('cookie monster!'))
        rsp = self.app.get(reverse('cookie_test'))
        self.assertContains(rsp, 'cookie monster!')


@skipIf(django.VERSION < (1, 10), 'MIDDLEWARE is added in Django 1.10')
@override_settings(MIDDLEWARE=[])
class MiddlewareTest(WebTest):

    def test_middleware_setting_name(self):
        self.assertEqual(
            self.middleware_setting_name,
            'MIDDLEWARE'
        )


@skipIf(django.VERSION < (1, 10) or django.VERSION >= (2, 0),
        'MIDDLEWARE is added in Django 1.10')
@override_settings(MIDDLEWARE=None, MIDDLEWARE_CLASSES=[])
class MiddlewareClassesTest(WebTest):

    def test_middleware_setting_name(self):
        self.assertEqual(
            self.middleware_setting_name,
            'MIDDLEWARE_CLASSES'
        )



from django.contrib.auth import authenticate

from rest_framework import authentication


class WebtestAuthentication(authentication.BaseAuthentication):
    """Bridge between webtest and django-rest-framework."""

    header = 'WEBTEST_USER'

    def authenticate(self, request):
        value = request.META.get(self.header)
        if value:
            account = authenticate(django_webtest_user=value)
            if account and account.is_active:
                return (account, None)



# -*- coding: utf-8 -*-

TRAVIS_CONF = '''
language: python

cache:
  directories:
    - $HOME/.cache/pip

services:
  - postgresql
before_script:
  - psql -c 'create database django_webtest_tests;' -U postgres

install:
  - pip install tox
script:
  - tox
matrix:
  include:
'''

if __name__ == '__main__':
    import subprocess
    import sys
    p = subprocess.check_output('tox -l', shell=True)
    if sys.version_info.major == 3:
        p = p.decode('utf-8')
    with open('.travis.yml', 'w') as fd:
        fd.write(TRAVIS_CONF)
        for env in p.split('\n'):
            env = env.strip()
            if env and env not in ('travis',):
                dist = None
                if env.startswith('pypy'):
                    py = 'pypy'
                else:
                    py = '{0}.{1}'.format(env[2], env[3])
                    if int(env[2] + env[3]) > 38:
                        # looks like psql dos not work on xenial... skip 3.7
                        # for now
                        continue
                        dist = 'xenial'
                fd.write('    - python: "{}"\n'.format(py))
                fd.write('      env: TOXENV={}\n'.format(env))
                if dist is not None:
                    fd.write('      dist: {}\n'.format(dist))



# -*- coding: utf-8 -*-
import copy

from django.conf import settings
from django.test.signals import template_rendered
from django.core.handlers.wsgi import WSGIHandler
from django.test import TestCase, TransactionTestCase
from django.test.client import store_rendered_templates

from functools import partial

try:
    from importlib import import_module
except ImportError:
    from django.utils.importlib import import_module

from django.core import signals
try:
    from django.db import close_old_connections
except ImportError:
    from django.db import close_connection
    close_old_connections = None
try:
    from django.core.servers.basehttp import (
            AdminMediaHandler as StaticFilesHandler)
except ImportError:
    from django.contrib.staticfiles.handlers import StaticFilesHandler

from webtest import TestApp
try:
    from webtest.utils import NoDefault
except ImportError:
    NoDefault = ''

from django_webtest.response import DjangoWebtestResponse
from django_webtest.compat import to_string, to_wsgi_safe_string


# sentinel to differentiate user=None / user param not given
_notgiven = object()


class DjangoTestApp(TestApp):
    response_class = DjangoWebtestResponse

    def __init__(self, *args, **kwargs):
        extra_environ = (kwargs.get('extra_environ') or {}).copy()
        extra_environ.setdefault('HTTP_HOST', 'testserver')
        kwargs['extra_environ'] = extra_environ
        super(DjangoTestApp, self).__init__(self.get_wsgi_handler(), *args, **kwargs)

    def get_wsgi_handler(self):
        return StaticFilesHandler(WSGIHandler())

    def set_user(self, user):
        """Update the user used by the app globall; pass None to unset."""
        if user is None and 'WEBTEST_USER' in self.extra_environ:
            del self.extra_environ['WEBTEST_USER']
        if user is not None:
            self.extra_environ = self._update_environ(self.extra_environ, user)

    def _update_environ(self, environ, user=_notgiven):
        environ = environ or {}

        if user is not _notgiven:
            if user is None:
                # We can't just delete the key here, the test request is built
                # from self.extra_environ + this environ, so the header defined
                # by set_user will be found in self.extra_environ.
                environ['WEBTEST_USER'] = ''
            else:
                username = _get_username(user)
                environ['WEBTEST_USER'] = to_wsgi_safe_string(username)

        return environ

    def do_request(self, req, status, expect_errors):
        # Django closes the database connection after every request;
        # this breaks the use of transactions in your tests.
        if close_old_connections is not None:  # Django 1.6+
            signals.request_started.disconnect(close_old_connections)
            signals.request_finished.disconnect(close_old_connections)
        else:  # Django < 1.6
            signals.request_finished.disconnect(close_connection)

        try:
            req.environ.setdefault('REMOTE_ADDR', '127.0.0.1')

            # is this a workaround for
            # https://code.djangoproject.com/ticket/11111 ?
            req.environ['REMOTE_ADDR'] = to_string(req.environ['REMOTE_ADDR'])
            req.environ['PATH_INFO'] = to_string(req.environ['PATH_INFO'])

            # Curry a data dictionary into an instance of the template renderer
            # callback function.
            data = {}
            on_template_render = partial(store_rendered_templates, data)
            template_rendered.connect(on_template_render)

            response = super(DjangoTestApp, self).do_request(req, status,
                                                             expect_errors)

            # Add any rendered template detail to the response.
            # If there was only one template rendered (the most likely case),
            # flatten the list to a single element.
            def flattend(detail):
                if len(data[detail]) == 1:
                    return data[detail][0]
                return data[detail]

            response.context = None
            response.template = None
            response.templates = data.get('templates', None)

            if data.get('context'):
                response.context = flattend('context')

            if data.get('template'):
                response.template = flattend('template')
            elif data.get('templates'):
                response.template = flattend('templates')

            response.__class__ = self.response_class
            return response
        finally:
            if close_old_connections:  # Django 1.6+
                signals.request_started.connect(close_old_connections)
                signals.request_finished.connect(close_old_connections)
            else:  # Django < 1.6
                signals.request_finished.connect(close_connection)

    def get(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        auto_follow = kwargs.pop('auto_follow', False)

        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        response = super(DjangoTestApp, self).get(url, *args, **kwargs)

        def is_redirect(r):
            return r.status_int >= 300 and r.status_int < 400
        while auto_follow and is_redirect(response):
            response = response.follow(**kwargs)

        return response

    def post(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        return super(DjangoTestApp, self).post(url, *args, **kwargs)

    def put(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        return super(DjangoTestApp, self).put(url, *args, **kwargs)

    def patch(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        return super(DjangoTestApp, self).patch(url, *args, **kwargs)

    def head(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        return super(DjangoTestApp, self).head(url, *args, **kwargs)

    def options(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        return super(DjangoTestApp, self).options(url, *args, **kwargs)

    def delete(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        return super(DjangoTestApp, self).delete(url, *args, **kwargs)

    def post_json(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        return super(DjangoTestApp, self).post_json(url, *args, **kwargs)

    def put_json(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        return super(DjangoTestApp, self).put_json(url, *args, **kwargs)

    def patch_json(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        return super(DjangoTestApp, self).patch_json(url, *args, **kwargs)

    def delete_json(self, url, *args, **kwargs):
        extra_environ = kwargs.get('extra_environ')
        user = kwargs.pop('user', _notgiven)
        kwargs['extra_environ'] = self._update_environ(extra_environ, user)
        return super(DjangoTestApp, self).delete_json(url, *args, **kwargs)

    @property
    def session(self):
        """
        Obtains the current session variables.
        """
        engine = import_module(settings.SESSION_ENGINE)
        cookie = self.cookies.get(settings.SESSION_COOKIE_NAME, None)
        if cookie:
            return engine.SessionStore(cookie)
        else:
            return {}

    def set_cookie(self, *args, **kwargs):
        self.extra_environ = self._update_environ(self.extra_environ)
        return super(DjangoTestApp, self).set_cookie(*args, **kwargs)


class WebTestMixin(object):
    extra_environ = {}
    csrf_checks = True
    setup_auth = True
    app_class = DjangoTestApp

    def _patch_settings(self):
        '''
        Patches settings to add support for django-webtest authorization
        and (optional) to disable CSRF checks.
        '''
        self._DEBUG_PROPAGATE_EXCEPTIONS = settings.DEBUG_PROPAGATE_EXCEPTIONS
        self._MIDDLEWARE = self.settings_middleware[:]
        self._AUTHENTICATION_BACKENDS = settings.AUTHENTICATION_BACKENDS[:]
        self._REST_FRAMEWORK = getattr(
            settings, 'REST_FRAMEWORK', {'DEFAULT_AUTHENTICATION_CLASSES': []})

        self.settings_middleware = list(self.settings_middleware)
        settings.AUTHENTICATION_BACKENDS = list(
            settings.AUTHENTICATION_BACKENDS)
        settings.REST_FRAMEWORK = copy.deepcopy(self._REST_FRAMEWORK)
        settings.DEBUG_PROPAGATE_EXCEPTIONS = True

        if not self.csrf_checks:
            self._disable_csrf_checks()

        if self.setup_auth:
            self._setup_auth()

    def _unpatch_settings(self):
        ''' Restores settings to before-patching state '''
        self.settings_middleware = self._MIDDLEWARE
        settings.AUTHENTICATION_BACKENDS = self._AUTHENTICATION_BACKENDS
        settings.DEBUG_PROPAGATE_EXCEPTIONS = self._DEBUG_PROPAGATE_EXCEPTIONS
        settings.REST_FRAMEWORK = self._REST_FRAMEWORK

    def _setup_auth(self):
        ''' Setups django-webtest authorization '''
        self._setup_auth_middleware()
        self._setup_auth_backend()
        self._setup_auth_class()

    def _disable_csrf_checks(self):
        disable_csrf_middleware = (
            'django_webtest.middleware.DisableCSRFCheckMiddleware')
        if disable_csrf_middleware not in self.settings_middleware:
            self.settings_middleware.insert(0, disable_csrf_middleware)

    def _setup_auth_middleware(self):
        webtest_auth_middleware = (
            'django_webtest.middleware.WebtestUserMiddleware')
        django_auth_middleware = (
            'django.contrib.auth.middleware.AuthenticationMiddleware')

        if django_auth_middleware not in self.settings_middleware:
            # There can be a custom AuthenticationMiddleware subclass or
            # replacement, we can't compute its index so just put our auth
            # middleware to the end.  If appending causes problems
            # _setup_auth_middleware method can be overriden by a subclass.
            self.settings_middleware.append(webtest_auth_middleware)
        elif webtest_auth_middleware not in self.settings_middleware:
            index = self.settings_middleware.index(django_auth_middleware)
            self.settings_middleware.insert(index + 1, webtest_auth_middleware)

    def _setup_auth_backend(self):
        backend_name = getattr(
            settings, 'WEBTEST_AUTHENTICATION_BACKEND',
            'django_webtest.backends.WebtestUserBackend')
        settings.AUTHENTICATION_BACKENDS.insert(0, backend_name)

    def _setup_auth_class(self):
        class_name = 'django_webtest.rest_framework_auth.WebtestAuthentication'
        drf_settings = settings.REST_FRAMEWORK
        try:
            classes = drf_settings['DEFAULT_AUTHENTICATION_CLASSES']
        except KeyError:
            classes = []
        if class_name not in classes:
            if isinstance(classes, tuple):
                classes = list(classes)
            classes.append(class_name)
            drf_settings['DEFAULT_AUTHENTICATION_CLASSES'] = classes

    @property
    def middleware_setting_name(self):
        try:
            return self._middleware_setting_name
        except AttributeError:
            if getattr(settings, 'MIDDLEWARE', None) is not None:
                name = 'MIDDLEWARE'
            else:
                name = 'MIDDLEWARE_CLASSES'
            self._middleware_setting_name = name
            return name

    @property
    def settings_middleware(self):
        return getattr(settings, self.middleware_setting_name)

    @settings_middleware.setter
    def settings_middleware(self, value):
        setattr(settings, self.middleware_setting_name, value)

    def renew_app(self):
        """
        Resets self.app (drops the stored state): cookies, etc.
        Note: this renews only self.app, not the responses fetched by self.app.
        """
        self.app = self.app_class(extra_environ=self.extra_environ)

    def __call__(self, result=None):
        self._patch_settings()
        self.renew_app()
        res = super(WebTestMixin, self).__call__(result)
        self._unpatch_settings()
        return res


class WebTest(WebTestMixin, TestCase):
    pass


class TransactionWebTest(WebTestMixin, TransactionTestCase):
    pass


def _get_username(user):
    """
    Return user's username. ``user`` can be standard Django User
    instance, a custom user model or just an username (as string).
    """
    value = None
    # custom user, django 1.5+
    get_username = getattr(user, 'get_username', None)
    if get_username is not None:
        value = get_username()
    if value is None:
        # standard User
        username = getattr(user, 'username', None)
        if username is not None:
            value = username
        else:
            # assume user is just an username
            value = user
    return value



#!/usr/bin/env python
import sys

from setuptools import setup


version = '1.9.13'


def _read(name):
    if sys.version_info[0] < 3:
        with open(name) as f:
            return f.read()
    else:
        with open(name, encoding='utf8') as f:
            return f.read()


def get_long_description():
    return _read('README.rst') + "\n\n" + _read('CHANGES.rst')


setup(
    name='django-webtest',
    version=version,
    author='Mikhail Korobov',
    author_email='kmike84@gmail.com',

    packages=['django_webtest'],

    url='https://github.com/django-webtest/django-webtest',
    license='MIT license',
    description=(
        "Instant integration of Ian Bicking's WebTest "
        "(http://docs.pylonsproject.org/projects/webtest/) "
        "with Django's testing framework."
    ),

    long_description=get_long_description(),
    install_requires=['webtest >= 1.3.3'],

    entry_points="""
    [pytest11]
    django_webtest = django_webtest.pytest_plugin
    """,

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 4.2',
        'Framework :: Django :: 5.0',
        'Framework :: Django :: 5.1',
        'Framework :: Django :: 5.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Testing',
    ],
    project_urls={
        'Code': 'https://github.com/django-webtest/django-webtest',
        'Issue Tracker': 'https://github.com/django-webtest/django-webtest/issues',
        'Changelog': 'https://github.com/django-webtest/django-webtest/blob/master/CHANGES.rst',
    },
    keywords=['django', 'webtest', 'pytest'],
)



import django
from django.http import HttpResponse
from django.shortcuts import render

from testapp_tests.views import check_password, search, set_session, \
    protected, redirect_to_protected, remove_prefix_redirect, cookie_test


major_version = int(django.__version__[0])

if major_version >= 4:
    from django.urls import re_path
else:
    try:
        from django.conf.urls import url
    except ImportError:
        from django.conf.urls.defaults import url


try:
    from django.contrib.auth.views import login
except ImportError:
    # django 2+
    from django.contrib.auth.views import LoginView
    login = LoginView.as_view()


def simple_method_test(request):
    return HttpResponse(str(request.method))


def simple_template_render(request, template_name):
    return render(request, template_name, {
        'user': request.user,
        'foo': ('a', 'b', 'c'),
        'bar': True,
        'spam': None,
    })


if major_version >= 4:
    urlpatterns = (
        re_path(r'$', simple_method_test, name='simple-method-test'),
        re_path(r'template/(.*)$', simple_template_render,
                name='simple-template-test'),
        re_path(r'check-password/$', check_password, name='check_password'),
        re_path(r'search/$', search, name='search'),
        re_path(r'login/$', login, name='auth_login'),
        re_path(r'set-session/$', set_session, name='set_session'),
        re_path(r'protected/$', protected, name='protected'),
        re_path(r'redirect-to-protected/$',
                redirect_to_protected, name='redirect-to-protected'),
        re_path(r'remove-prefix-redirect/(.*)/$',
                remove_prefix_redirect, name='remove-prefix-redirect'),
        re_path(r'cookie-test/$', cookie_test, name='cookie_test'),
    )
else:
    urlpatterns = (
        url(r'^$', simple_method_test, name='simple-method-test'),
        url(r'^template/(.*)$', simple_template_render,
            name='simple-template-test'),
        url(r'^check-password/$', check_password, name='check_password'),
        url(r'^search/$', search, name='search'),
        url(r'^login/$', login, name='auth_login'),
        url(r'^set-session/$', set_session, name='set_session'),
        url(r'^protected/$', protected, name='protected'),
        url(r'^redirect-to-protected/$',
            redirect_to_protected, name='redirect-to-protected'),
        url(r'^remove-prefix-redirect/(.*)/$',
            remove_prefix_redirect, name='remove-prefix-redirect'),
        url(r'^cookie-test/$', cookie_test, name='cookie_test'),
    )



import sys
import urllib

import django

PY3 = sys.version_info[0] == 3

if PY3:
    from urllib import parse as urlparse

    def to_string(s):
        if isinstance(s, str):
            return s
        return str(s, 'latin1')

    def to_wsgi_safe_string(s):
        return urlparse.quote(to_string(s))

    def from_wsgi_safe_string(s):
        return urlparse.unquote(s)

else:
    import urlparse

    def to_string(s):
        return str(s)

    def to_wsgi_safe_string(s):
        return to_string(urllib.quote(s.encode('utf8')))

    def from_wsgi_safe_string(s):
        return urllib.unquote(s).decode('utf8')


def is_authenticated(user):
    if django.VERSION < (1, 10):
        return user.is_authenticated()
    return user.is_authenticated


def is_anonymous(user):
    if django.VERSION < (1, 10):
        return user.is_anonymous()
    return user.is_anonymous



from django import forms
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render, redirect

class PasswordForm(forms.Form):
    password = forms.CharField(widget=forms.PasswordInput())

    def clean_password(self):
        if self.cleaned_data['password'] != 'foo':
            raise forms.ValidationError('Incorrect password.')
        return self.cleaned_data['password']

def check_password(request):
    form = PasswordForm(request.POST or None)
    if form.is_valid():
        return HttpResponseRedirect('/')
    return render(request, 'form.html', {'form': form})


class SearchForm(forms.Form):
    q = forms.CharField(required=False)

def search(request):
    form = SearchForm(request.GET)
    q = None
    if form.is_valid():
        q = form.cleaned_data['q']
    return render(request, 'get_form.html', {'form': form, 'q': q})

def set_session(request):
    request.session['test'] = 'foo'
    return HttpResponseRedirect('/')

def redirect_to_protected(request):
    return redirect('protected')

@login_required
def protected(request):
    return HttpResponse('ok: {0}'.format(request.user.username))

def remove_prefix_redirect(request, arg):
    return HttpResponseRedirect("/" + arg)

def cookie_test(request):
    cookie = request.COOKIES.get(str('test_cookie'), None)
    return HttpResponse('cookie: {0}'.format(cookie))
